﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Praktikum_1
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void ButtonLade_Click(object sender, RoutedEventArgs e)
        {
            // BinaryReader usw. brauchen: using System.IO
            // Pfad zur Datei mit den Messdaten anpassen!
            BinaryReader br = new BinaryReader(File.OpenRead(@"..\..\..\..\netzspannung.raw"));
            
            int anzahlSamples = (int)(br.BaseStream.Length / sizeof(double));
            double[] samples = new double[anzahlSamples];

            for (int i = 0; i < anzahlSamples; i++)
			{
			    samples[i] = br.ReadDouble();
			}

            // Alte Art:
            //double maximum = double.MinValue;
            //double minimum = double.MaxValue;
            //double summeQuadrate = 0.0;
            //for (int i = 0; i < anzahlSamples; i++)
            //{
            //    double s = samples[i];
            //    maximum = Math.Max(maximum, s);
            //    minimum = Math.Min(minimum, s);
            //    summeQuadrate += s * s;
            //}
            //double effektivwert = Math.Sqrt(summeQuadrate / anzahlSamples);

            // Kürzer und moderner:
            double maximum = samples.Max();
            double minimum = samples.Min();
            double effektivwert = Math.Sqrt(samples.Average(x => x * x));

            int anzahlNulldurchgänge = 0;
            int indexErsterNulldurchgang = -1;
            bool erstenNullDurchgangGefunden = false;
            int indexLetzterNulldurchgang = -1;
            for (int i = 0; i < anzahlSamples - 1; i++)
            {
                double s = samples[i];
                double s1 = samples[i + 1];
                if(s * s1 < 0.0)
                {
                    anzahlNulldurchgänge++;
                    if(!erstenNullDurchgangGefunden)
                    {
                        erstenNullDurchgangGefunden = true;
                        indexErsterNulldurchgang = i;
                    }
                    indexLetzterNulldurchgang = i;
                }
            }
            const int abtastrate = 48000;
            double frequenz = abtastrate * 0.5 * (anzahlNulldurchgänge - 1) / (indexLetzterNulldurchgang - indexErsterNulldurchgang);

            textBlockAusgabe.Text = "Minimum = " + minimum.ToString("#.# V")
                + "\nMaximum = " + maximum.ToString("#.# V")
                + "\nEffektivwert = " + effektivwert.ToString("#.# V")
                + "\nFrequenz = " + frequenz.ToString("#.### Hz");

            double skalaY = Math.Max(Math.Abs(minimum), Math.Abs(maximum));
            double skalierungsfaktor = 0.5 * zeichenfläche.ActualHeight / skalaY;

            const double intervallTeilstricheX = 0.1;
            int anzahlTeilstricheX = (int)(anzahlSamples / (abtastrate * intervallTeilstricheX) + 1);
            for (int k = 0; k < anzahlTeilstricheX; k++)
            {
                Line linie = new Line();
                linie.X1 = linie.X2 = zeichenfläche.Width * k / anzahlSamples * abtastrate * intervallTeilstricheX;
                linie.Y1 = 0.0;
                linie.Y2 = zeichenfläche.ActualHeight;
                linie.Stroke = Brushes.White;
                linie.StrokeThickness = 2.0;
                zeichenfläche.Children.Add(linie);
            }

            const double intervallTeilstricheY = 100.0;
            int anzahlPositiveTeilstricheY = (int)(skalaY/intervallTeilstricheY);
            for (int k = -anzahlPositiveTeilstricheY; k <= anzahlPositiveTeilstricheY; k++)
            {
                Line linie = new Line();
                linie.X1 = 0.0;
                linie.X2 = zeichenfläche.Width;
                linie.Y1 = linie.Y2 = 0.5 * zeichenfläche.ActualHeight - k * intervallTeilstricheY * skalierungsfaktor;
                linie.Stroke = Brushes.White;
                linie.StrokeThickness = 2.0;
                zeichenfläche.Children.Add(linie);
            }

            double[] linienHöhen = { -effektivwert, effektivwert };
            for (int j = 0; j < linienHöhen.Length; j++)
            {
                Line linie = new Line();
                linie.X1 = 0;
                linie.X2 = zeichenfläche.ActualWidth;
                linie.Y1 = linie.Y2 = 0.5 * zeichenfläche.ActualHeight - linienHöhen[j] * skalierungsfaktor;
                linie.Stroke = Brushes.Red;
                linie.StrokeThickness = 1.0;
                zeichenfläche.Children.Add(linie);
            }

            Polyline pl = new Polyline();
            for (int i = 0; i < anzahlSamples - 1; i++)
            {
                double x = zeichenfläche.ActualWidth * i / anzahlSamples; // Vorsicht: i / anzahlSamples * zeichenfläche.ActualWidth würde 0 ergeben.
                double y = 0.5 * zeichenfläche.ActualHeight - samples[i] * skalierungsfaktor;
                pl.Points.Add(new Point(x, y));
            }
            pl.Stroke = Brushes.Blue;
            pl.StrokeThickness = 1.0;
            pl.StrokeLineJoin = PenLineJoin.Round;
            zeichenfläche.Children.Add(pl);
        }
    }
}
